package middleware

import (
	"net/http"
	"runtime/debug"

	"github.com/go-kit/log"
	"github.com/go-kit/log/level"
)

// Recoverer initializes a recoverer middleware.
func Recoverer(logger log.Logger) func(next http.Handler) http.Handler {
	return func(next http.Handler) http.Handler {
		fn := func(w http.ResponseWriter, r *http.Request) {
			defer func() {
				if rvr := recover(); rvr != nil {
					level.Error(logger).Log(
						"msg", rvr.(string),
						"trace", string(debug.Stack()),
					)

					http.Error(
						w,
						http.StatusText(http.StatusInternalServerError),
						http.StatusInternalServerError,
					)
				}
			}()

			next.ServeHTTP(w, r)
		}

		return http.HandlerFunc(fn)
	}
}
